@@ -1,4 +1,5 @@
-/* Copyright (C) 2000-2004 MySQL AB
+
+/* Copyright (C) 2000-2005 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -242,18 +243,25 @@ char *SQLTABLES_type_values[2][5]=
   {NULL,NULL,NULL,"SYSTEM TABLE",NULL}, /* TODO: add temp tables here */
 };
 
+#if MYSQL_VERSION_ID >= 40100
 MYSQL_FIELD SQLTABLES_fields[]=
 {
-  {"TABLE_CAT","Catalog", NullS, NullS, NullS, NAME_LEN, 0, 0, 0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_SCHEM","Catalog",NullS, NullS, NullS, NAME_LEN, 0, 0, 0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_NAME","Catalog", NullS, NullS, NullS, NAME_LEN, NAME_LEN, 0, 0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_TYPE","Catalog",NullS,NullS, NullS, NAME_LEN, 5, 0, 0,
-   FIELD_TYPE_VAR_STRING},
-  {"REMARKS","Catalog",NullS,NullS, NullS, NAME_LEN, 11, 0, 0,
-   FIELD_TYPE_VAR_STRING}};
+  {"TABLE_CAT",     NullS,"Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_SCHEM",   NullS,"Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_NAME",    NullS,"Catalog",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_TYPE",    NullS,"Catalog",NullS,NullS,NullS,NullS,NAME_LEN,5, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"REMARKS",       NullS,"Catalog",NullS,NullS,NullS,NullS,NAME_LEN,11, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING}
+};
+#else
+MYSQL_FIELD SQLTABLES_fields[]=
+{
+  {"TABLE_CAT",     "Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_SCHEM",   "Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_NAME",    "Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_TYPE",    "Catalog",NullS,NullS,NullS,NAME_LEN,5,0,0,FIELD_TYPE_VAR_STRING},
+  {"REMARKS",       "Catalog",NullS,NullS,NullS,NAME_LEN,11,0,0,FIELD_TYPE_VAR_STRING}
+};
+#endif
 
 const uint SQLTABLES_FIELDS= array_elements(SQLTABLES_values);
 
@@ -281,12 +289,6 @@ SQLRETURN SQL_API SQLTables(SQLHSTMT    hstmt,
 
   DBUG_ENTER("SQLTables");
 
-  DBUG_PRINT("enter",("Qualifier: '%s'(%d)  Owner: '%s'(%d)  Table: '%s'(%d)  Type: '%s'(%d)",
-          szTableQualifier ? (char*) szTableQualifier : "null", cbTableQualifier,
-          szTableOwner ? (char*) szTableOwner : "null", cbTableOwner,
-          szTableName ? (char*) szTableName : "null", cbTableName,
-          szTableType ? (char*) szTableType : "null", cbTableType));
-
   CLEAR_STMT_ERROR(hstmt);
   my_SQLFreeStmt(hstmt,MYSQL_RESET);
 
@@ -563,40 +565,51 @@ char *SQLCOLUMNS_values[]= {
   SC_coldef,SC_sqltype,NullS,SC_octlen,NullS,SC_isnull
 };
 
+#if MYSQL_VERSION_ID >= 40100
 MYSQL_FIELD SQLCOLUMNS_fields[]=
 {
-  {"TABLE_CAT","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_SCHEM","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_NAME","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,
-   NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
-  {"COLUMN_NAME","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,
-   NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
-  {"DATA_TYPE","MySQL_Catalog",NullS,NullS,NullS,5,5,NOT_NULL_FLAG,0,
-   FIELD_TYPE_SHORT},
-  {"TYPE_NAME","MySQL_Catalog",NullS,NullS,NullS,20,20,NOT_NULL_FLAG,0,
-   FIELD_TYPE_VAR_STRING},
-  {"COLUMN_SIZE","MySQL_Catalog",NullS,NullS,NullS,11,11,0,0,FIELD_TYPE_LONG},
-  {"BUFFER_LENGTH","MySQL_Catalog",NullS,NullS,NullS,11,11,0,0,
-   FIELD_TYPE_LONG},
-  {"DECIMAL_DIGITS","MySQL_Catalog",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
-  {"NUM_PREC_RADIX","MySQL_Catalog",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
-  {"NULLABLE","MySQL_Catalog",NullS,NullS,NullS,5,5,NOT_NULL_FLAG,0,
-   FIELD_TYPE_SHORT},
-  {"REMARKS","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"COLUMN_DEF","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"SQL_DATA_TYPE","MySQL_Catalog",NullS,NullS,NullS,5,5,NOT_NULL_FLAG,0,
-   FIELD_TYPE_SHORT},
-  {"SQL_DATETIME_SUB","MySQL_Catalog",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
-  {"CHAR_OCTET_LENGTH","MySQL_Catalog",NullS,NullS,NullS,11,11,0,0,
-   FIELD_TYPE_LONG},
-  {"ORDINAL_POSITION","MySQL_Catalog",NullS,NullS,NullS,11,11,NOT_NULL_FLAG,0,
-   FIELD_TYPE_LONG},
-  {"IS_NULLABLE","MySQL_Catalog",NullS,NullS,NullS,3,3,0,0,
-   FIELD_TYPE_VAR_STRING}};
+  {"TABLE_CAT",         NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_SCHEM",       NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_NAME",        NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"COLUMN_NAME",       NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"DATA_TYPE",         NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,5,5, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"TYPE_NAME",         NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,20,20, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"COLUMN_SIZE",       NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,11,11, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_LONG},
+  {"BUFFER_LENGTH",     NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,11,11, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_LONG},
+  {"DECIMAL_DIGITS",    NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
+  {"NUM_PREC_RADIX",    NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
+  {"NULLABLE",          NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,5,5, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"REMARKS",           NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"COLUMN_DEF",        NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"SQL_DATA_TYPE",     NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,5,5, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"SQL_DATETIME_SUB",  NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
+  {"CHAR_OCTET_LENGTH", NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,11,11, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_LONG},
+  {"ORDINAL_POSITION",  NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,11,11, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_LONG},
+  {"IS_NULLABLE",       NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,3,3, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING}
+};
+#else
+MYSQL_FIELD SQLCOLUMNS_fields[]=
+{
+  {"TABLE_CAT",         "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_SCHEM",       "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_NAME",        "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
+  {"COLUMN_NAME",       "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
+  {"DATA_TYPE",         "MySQL_Catalog",NullS,NullS,NullS,5,5,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"TYPE_NAME",         "MySQL_Catalog",NullS,NullS,NullS,20,20,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
+  {"COLUMN_SIZE",       "MySQL_Catalog",NullS,NullS,NullS,11,11,0,0,FIELD_TYPE_LONG},
+  {"BUFFER_LENGTH",     "MySQL_Catalog",NullS,NullS,NullS,11,11,0,0,FIELD_TYPE_LONG},
+  {"DECIMAL_DIGITS",    "MySQL_Catalog",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
+  {"NUM_PREC_RADIX",    "MySQL_Catalog",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
+  {"NULLABLE",          "MySQL_Catalog",NullS,NullS,NullS,5,5,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"REMARKS",           "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,0,0,FIELD_TYPE_VAR_STRING},
+  {"COLUMN_DEF",        "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,0,0,FIELD_TYPE_VAR_STRING},
+  {"SQL_DATA_TYPE",     "MySQL_Catalog",NullS,NullS,NullS,5,5,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"SQL_DATETIME_SUB",  "MySQL_Catalog",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
+  {"CHAR_OCTET_LENGTH", "MySQL_Catalog",NullS,NullS,NullS,11,11,0,0,FIELD_TYPE_LONG},
+  {"ORDINAL_POSITION",  "MySQL_Catalog",NullS,NullS,NullS,11,11,NOT_NULL_FLAG,0,FIELD_TYPE_LONG},
+  {"IS_NULLABLE",       "MySQL_Catalog",NullS,NullS,NullS,3,3,0,0,FIELD_TYPE_VAR_STRING}
+};
+#endif
 
 const uint SQLCOLUMNS_FIELDS= array_elements(SQLCOLUMNS_values);
 
@@ -656,7 +669,7 @@ static MYSQL_RES* mysql_list_dbcolumns(STMT FAR   *stmt,
       strxmov(select,select," FROM ",TableQualifier,".`",TableName,"`",NullS);  
     else      
       strxmov(select,select," FROM ",TableName,NullS);  
-    
+    strxmov(select,select," LIMIT 0",NullS);
     mysql_free_result(result);
 
     MYLOG_QUERY(stmt, select);
@@ -853,32 +866,41 @@ char SS_type[10];
 uint SQLSTAT_order[]={2,3,5,7,8,9,10};
 char *SQLSTAT_values[]={NullS,NullS,"","",NullS,"",SS_type,"","","","",NullS,NullS};
 
+#if MYSQL_VERSION_ID >= 40100
 MYSQL_FIELD SQLSTAT_fields[]=
 {
-  {"TABLE_CAT","MySQL_Stat",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_SCHEM","MySQL_Stat",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_NAME","MySQL_Stat",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,
-   0, FIELD_TYPE_VAR_STRING},
-  {"NON_UNIQUE","MySQL_Stat",NullS,NullS,NullS,1,1,NOT_NULL_FLAG,0,
-   FIELD_TYPE_SHORT},
-  {"INDEX_QUALIFIER","MySQL_Stat",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"INDEX_NAME","MySQL_Stat",NullS,NullS,NullS,NAME_LEN,NAME_LEN,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"TYPE","MySQL_Stat",NullS,NullS,NullS,1,1,NOT_NULL_FLAG,0,
-   FIELD_TYPE_SHORT},
-  {"ORDINAL_POSITION","MySQL_Stat",NullS,NullS,NullS,1,2,NOT_NULL_FLAG,0,
-   FIELD_TYPE_SHORT},
-  {"COLUMN_NAME","MySQL_Stat",NullS,NullS,NullS,NAME_LEN,NAME_LEN,
-   NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
-  {"ASC_OR_DESC","MySQL_Stat",NullS,NullS,NullS,1,1,0,0,FIELD_TYPE_VAR_STRING},
-  {"CARDINALITY","MySQL_Stat",NullS,NullS,NullS,11,11,0,0,FIELD_TYPE_LONG},
-  {"PAGES","MySQL_Stat",NullS,NullS,NullS,9,9,0,0,FIELD_TYPE_LONG},
-  {"FILTER_CONDITION","MySQL_Stat",NullS,NullS,NullS,10,10,0,0,
-   FIELD_TYPE_VAR_STRING},
+  {"TABLE_CAT",         NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_SCHEM",       NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_NAME",        NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"NON_UNIQUE",        NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,1,1, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"INDEX_QUALIFIER",   NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"INDEX_NAME",        NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TYPE",              NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,1,1, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"ORDINAL_POSITION",  NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,1,2, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"COLUMN_NAME",       NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"ASC_OR_DESC",       NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,1,1, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"CARDINALITY",       NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,11,11, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_LONG},
+  {"PAGES",             NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,9,9, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_LONG},
+  {"FILTER_CONDITION",  NullS,"MySQL_Stat",NullS,NullS,NullS,NullS,10,10, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
 };
+#else
+MYSQL_FIELD SQLSTAT_fields[]=
+{
+  {"TABLE_CAT",         "MySQL_Stat",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_SCHEM",       "MySQL_Stat",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_NAME",        "MySQL_Stat",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,0, FIELD_TYPE_VAR_STRING},
+  {"NON_UNIQUE",        "MySQL_Stat",NullS,NullS,NullS,1,1,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"INDEX_QUALIFIER",   "MySQL_Stat",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"INDEX_NAME",        "MySQL_Stat",NullS,NullS,NullS,NAME_LEN,NAME_LEN,0,0,FIELD_TYPE_VAR_STRING},
+  {"TYPE",              "MySQL_Stat",NullS,NullS,NullS,1,1,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"ORDINAL_POSITION",  "MySQL_Stat",NullS,NullS,NullS,1,2,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"COLUMN_NAME",       "MySQL_Stat",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
+  {"ASC_OR_DESC",       "MySQL_Stat",NullS,NullS,NullS,1,1,0,0,FIELD_TYPE_VAR_STRING},
+  {"CARDINALITY",       "MySQL_Stat",NullS,NullS,NullS,11,11,0,0,FIELD_TYPE_LONG},
+  {"PAGES",             "MySQL_Stat",NullS,NullS,NullS,9,9,0,0,FIELD_TYPE_LONG},
+  {"FILTER_CONDITION",  "MySQL_Stat",NullS,NullS,NullS,10,10,0,0,FIELD_TYPE_VAR_STRING},
+};
+#endif
 
 const uint SQLSTAT_FIELDS= array_elements(SQLSTAT_fields);
 
@@ -957,7 +979,11 @@ SQLRETURN SQL_API SQLStatistics(SQLHSTMT hstmt,
     goto empty_set;
   }
   pthread_mutex_unlock(&dbc->lock);
+#if MYSQL_VERSION_ID >= 40100
+  int2str(SQL_INDEX_OTHER,SS_type,10,0);
+#else
   int2str(SQL_INDEX_OTHER,SS_type,10);
+#endif
   stmt->order=       SQLSTAT_order;
   stmt->order_count= array_elements(SQLSTAT_order);
   stmt->fix_fields=  fix_fields_copy;
@@ -1070,23 +1096,29 @@ char *SQLTABLES_priv_values[]=
   NULL,"",NULL,NULL,NULL,NULL,NULL
 };
 
+#if MYSQL_VERSION_ID >= 40100
 MYSQL_FIELD SQLTABLES_priv_fields[]=
 {
-  {"TABLE_CAT","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_SCHEM","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_NAME","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,
-   NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
-  {"GRANTOR","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"GRANTEE","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,
-   0,FIELD_TYPE_VAR_STRING},
-  {"PRIVILEGE","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,
-   NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
-  {"IS_GRANTABLE","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
+  {"TABLE_CAT",     NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_SCHEM",   NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_NAME",    NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"GRANTOR",       NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"GRANTEE",       NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"PRIVILEGE",     NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"IS_GRANTABLE",  NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
 };
+#else
+MYSQL_FIELD SQLTABLES_priv_fields[]=
+{
+  {"TABLE_CAT",     "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_SCHEM",   "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_NAME",    "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
+  {"GRANTOR",       "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"GRANTEE",       "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
+  {"PRIVILEGE",     "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
+  {"IS_GRANTABLE",  "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+};
+#endif
 
 const uint SQLTABLES_PRIV_FIELDS= array_elements(SQLTABLES_priv_values);
 
@@ -1233,25 +1265,31 @@ char *SQLCOLUMNS_priv_values[]=
   NULL,"",NULL,NULL,NULL,NULL,NULL,NULL
 };
 
+#if MYSQL_VERSION_ID >= 40100
 MYSQL_FIELD SQLCOLUMNS_priv_fields[]=
 {
-  {"TABLE_CAT","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_SCHEM","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_NAME","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,
-   NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
-  {"COLUMN_NAME","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,
-   NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
-  {"GRANTOR","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"GRANTEE","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,
-   0,FIELD_TYPE_VAR_STRING},
-  {"PRIVILEGE","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,
-   NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
-  {"IS_GRANTABLE","MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
+  {"TABLE_CAT",     NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_SCHEM",   NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_NAME",    NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"COLUMN_NAME",   NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"GRANTOR",       NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"GRANTEE",       NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"PRIVILEGE",     NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"IS_GRANTABLE",  NullS,"MySQL_Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+};                                                           
+#else
+MYSQL_FIELD SQLCOLUMNS_priv_fields[]=
+{
+  {"TABLE_CAT",     "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_SCHEM",   "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_NAME",    "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
+  {"COLUMN_NAME",   "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
+  {"GRANTOR",       "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"GRANTEE",       "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
+  {"PRIVILEGE",     "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
+  {"IS_GRANTABLE",  "MySQL_Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
 };
+#endif
 
 const uint SQLCOLUMNS_PRIV_FIELDS= array_elements(SQLCOLUMNS_priv_values);
 
@@ -1366,25 +1404,31 @@ SQLSpecialColumns
 ****************************************************************************
 */
 
+#if MYSQL_VERSION_ID >= 40100
 MYSQL_FIELD SQLSPECIALCOLUMNS_fields[]=
 {
-  {"SCOPE","MySQL_SpecialColumns",NullS,NullS,NullS,5,5,
-   NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
-  {"COLUMN_NAME","MySQL_SpecialColumns",NullS,NullS,NullS,NAME_LEN,
-   NAME_LEN,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
-  {"DATA_TYPE","MySQL_SpecialColumns",NullS,NullS,NullS,5,5,
-   NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
-  {"TYPE_NAME","MySQL_SpecialColumns",NullS,NullS,NullS,20,20,
-   NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
-  {"COLUMN_SIZE","MySQL_SpecialColumns",NullS,NullS,NullS,7,7,0,0,
-   FIELD_TYPE_LONG},
-  {"BUFFER_LENGTH","MySQL_SpecialColumns",NullS,NullS,NullS,7,7,0,0,
-   FIELD_TYPE_LONG},
-  {"DECIMAL_DIGITS","MySQL_SpecialColumns",NullS,NullS,NullS,3,3,0,0,
-   FIELD_TYPE_SHORT},
-  {"PSEUDO_COLUMN","MySQL_SpecialColumns",NullS,NullS,NullS,3,3,0,0,
-   FIELD_TYPE_SHORT}
+  {"SCOPE",         NullS,"MySQL_SpecialColumns",NullS,NullS,NullS,NullS,5,5, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"COLUMN_NAME",   NullS,"MySQL_SpecialColumns",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"DATA_TYPE",     NullS,"MySQL_SpecialColumns",NullS,NullS,NullS,NullS,5,5, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"TYPE_NAME",     NullS,"MySQL_SpecialColumns",NullS,NullS,NullS,NullS,20,20, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"COLUMN_SIZE",   NullS,"MySQL_SpecialColumns",NullS,NullS,NullS,NullS,7,7, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_LONG},
+  {"BUFFER_LENGTH", NullS,"MySQL_SpecialColumns",NullS,NullS,NullS,NullS,7,7, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_LONG},
+  {"DECIMAL_DIGITS",NullS,"MySQL_SpecialColumns",NullS,NullS,NullS,NullS,3,3, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
+  {"PSEUDO_COLUMN", NullS,"MySQL_SpecialColumns",NullS,NullS,NullS,NullS,3,3, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT}
 };
+#else
+MYSQL_FIELD SQLSPECIALCOLUMNS_fields[]=
+{
+  {"SCOPE",         "MySQL_SpecialColumns",NullS,NullS,NullS,5,5,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"COLUMN_NAME",   "MySQL_SpecialColumns",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
+  {"DATA_TYPE",     "MySQL_SpecialColumns",NullS,NullS,NullS,5,5,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"TYPE_NAME",     "MySQL_SpecialColumns",NullS,NullS,NullS,20,20,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
+  {"COLUMN_SIZE",   "MySQL_SpecialColumns",NullS,NullS,NullS,7,7,0,0,FIELD_TYPE_LONG},
+  {"BUFFER_LENGTH", "MySQL_SpecialColumns",NullS,NullS,NullS,7,7,0,0,FIELD_TYPE_LONG},
+  {"DECIMAL_DIGITS","MySQL_SpecialColumns",NullS,NullS,NullS,3,3,0,0,FIELD_TYPE_SHORT},
+  {"PSEUDO_COLUMN", "MySQL_SpecialColumns",NullS,NullS,NullS,3,3,0,0,FIELD_TYPE_SHORT}
+};
+#endif
 
 char *SQLSPECIALCOLUMNS_values[]= {
   0,NULL,0,NULL,0,0,0,0
@@ -1584,21 +1628,27 @@ SQLPrimaryKeys
 ****************************************************************************
 */
 
+#if MYSQL_VERSION_ID >= 40100
 MYSQL_FIELD SQLPRIM_KEYS_fields[]=
 {
-  {"TABLE_CAT","MySQL_Primary_keys",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_SCHEM","MySQL_Primary_keys",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"TABLE_NAME","MySQL_Primary_keys",NullS,NullS,NullS,NAME_LEN,
-   NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
-  {"COLUMN_NAME","MySQL_Primary_keys",NullS,NullS,NullS,NAME_LEN,
-   NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
-  {"KEY_SEQ","MySQL_Primary_keys",NullS,NullS,NullS,2,2,NOT_NULL_FLAG,0,
-   FIELD_TYPE_SHORT},
-  {"PK_NAME","MySQL_Primary_keys",NullS,NullS,NullS,128,0,0,0,
-   FIELD_TYPE_VAR_STRING},
+  {"TABLE_CAT",     NullS,"MySQL_Primary_keys",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_SCHEM",   NullS,"MySQL_Primary_keys",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_NAME",    NullS,"MySQL_Primary_keys",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN,  0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"COLUMN_NAME",   NullS,"MySQL_Primary_keys",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN,  0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"KEY_SEQ",       NullS,"MySQL_Primary_keys",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"PK_NAME",       NullS,"MySQL_Primary_keys",NullS,NullS,NullS,NullS,128,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
 };
+#else
+MYSQL_FIELD SQLPRIM_KEYS_fields[]=
+{
+  {"TABLE_CAT",     "MySQL_Primary_keys",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_SCHEM",   "MySQL_Primary_keys",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"TABLE_NAME",    "MySQL_Primary_keys",NullS,NullS,NullS,NAME_LEN,NAME_LEN,NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
+  {"COLUMN_NAME",   "MySQL_Primary_keys",NullS,NullS,NullS,NAME_LEN,NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
+  {"KEY_SEQ",       "MySQL_Primary_keys",NullS,NullS,NullS,2,2,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"PK_NAME",       "MySQL_Primary_keys",NullS,NullS,NullS,128,0,0,0,FIELD_TYPE_VAR_STRING},
+};
+#endif
 
 const uint SQLPRIM_KEYS_FIELDS= array_elements(SQLPRIM_KEYS_fields);
 
@@ -1696,38 +1746,43 @@ empty_set:
 SQLForeignJeys
 ****************************************************************************
 */
-
+#if MYSQL_VERSION_ID >= 40100
 MYSQL_FIELD SQLFORE_KEYS_fields[]=
 {
-  {"PKTABLE_CAT","MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"PKTABLE_SCHEM","MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"PKTABLE_NAME","MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,
-   NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
-  {"PKCOLUMN_NAME","MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,
-   NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
-  {"FKTABLE_CAT","MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"FKTABLE_SCHEM","MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,
-   NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
-  {"FKTABLE_NAME","MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,
-   NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
-  {"FKCOLUMN_NAME","MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,
-   NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
-  {"KEY_SEQ","MySQL_Foreign_keys",NullS,NullS,NullS,2,2,NOT_NULL_FLAG,0,
-   FIELD_TYPE_SHORT},
-  {"UPDATE_RULE","MySQL_Foreign_keys",NullS,NullS,NullS,2,2,0,0,
-   FIELD_TYPE_SHORT},
-  {"DELETE_RULE","MySQL_Foreign_keys",NullS,NullS,NullS,2,2,0,0,
-   FIELD_TYPE_SHORT},
-  {"FK_NAME","MySQL_Foreign_keys",NullS,NullS,NullS,128,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"PK_NAME","MySQL_Foreign_keys",NullS,NullS,NullS,128,0,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"DEFERRABILITY","MySQL_Foreign_keys",NullS,NullS,NullS,2,2,0,0,
-   FIELD_TYPE_SHORT},
+  {"PKTABLE_CAT",   NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"PKTABLE_SCHEM", NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"PKTABLE_NAME",  NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"PKCOLUMN_NAME", NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"FKTABLE_CAT",   NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"FKTABLE_SCHEM", NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"FKTABLE_NAME",  NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"FKCOLUMN_NAME", NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,NAME_LEN,NAME_LEN, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"KEY_SEQ",       NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"UPDATE_RULE",   NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
+  {"DELETE_RULE",   NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
+  {"FK_NAME",       NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,128,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"PK_NAME",       NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,128,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"DEFERRABILITY", NullS,"MySQL_Foreign_keys",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
 };
+#else
+MYSQL_FIELD SQLFORE_KEYS_fields[]=
+{
+  {"PKTABLE_CAT",   "MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"PKTABLE_SCHEM", "MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"PKTABLE_NAME",  "MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
+  {"PKCOLUMN_NAME", "MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
+  {"FKTABLE_CAT",   "MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"FKTABLE_SCHEM", "MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
+  {"FKTABLE_NAME",  "MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
+  {"FKCOLUMN_NAME", "MySQL_Foreign_keys",NullS,NullS,NullS,NAME_LEN,NAME_LEN, NOT_NULL_FLAG, 0, FIELD_TYPE_VAR_STRING},
+  {"KEY_SEQ",       "MySQL_Foreign_keys",NullS,NullS,NullS,2,2,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"UPDATE_RULE",   "MySQL_Foreign_keys",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
+  {"DELETE_RULE",   "MySQL_Foreign_keys",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
+  {"FK_NAME",       "MySQL_Foreign_keys",NullS,NullS,NullS,128,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"PK_NAME",       "MySQL_Foreign_keys",NullS,NullS,NullS,128,0,0,0,FIELD_TYPE_VAR_STRING},
+  {"DEFERRABILITY", "MySQL_Foreign_keys",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
+};
+#endif
 
 const uint SQLFORE_KEYS_FIELDS= array_elements(SQLFORE_KEYS_fields);
 
@@ -1983,6 +2038,19 @@ static MYSQL_RES *mysql_list_sysprocs(DBC FAR *dbc,
 
 char *SQLPROCEDURES_values[]= {"","",NULL,0,0,0,"","procedure"};
 
+#if MYSQL_VERSION_ID >= 40100
+MYSQL_FIELD SQLPROCEDURES_fields[]= 
+{
+  {"PROCEDURE_CAT",    NullS,"Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"PROCEDURE_SCHEM",  NullS,"Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"PROCEDURE_NAME",   NullS,"Catalog",NullS,NullS,NullS,NullS,NAME_LEN,0, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"NUM_INPUT_PARAMS", NullS,"Catalog",NullS,NullS,NullS,NullS,11,11, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_LONG},
+  {"NUM_OUTPUT_PARAMS",NullS,"Catalog",NullS,NullS,NullS,NullS,11,11, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_LONG},
+  {"NUM_RESULT_SETS",  NullS,"Catalog",NullS,NullS,NullS,NullS,11,11, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_LONG},
+  {"REMARKS",          NullS,"Catalog",NullS,NullS,NullS,NullS,NAME_LEN,11, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"PROCEDURE_TYPE",   NullS,"Catalog",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT}
+};
+#else
 MYSQL_FIELD SQLPROCEDURES_fields[]= 
 {
   {"PROCEDURE_CAT",    "Catalog",NullS,NullS,NullS,NAME_LEN,0,0,0,FIELD_TYPE_VAR_STRING},
@@ -1991,9 +2059,10 @@ MYSQL_FIELD SQLPROCEDURES_fields[]=
   {"NUM_INPUT_PARAMS", "Catalog",NullS,NullS,NullS,11,11,0,0,FIELD_TYPE_LONG},
   {"NUM_OUTPUT_PARAMS","Catalog",NullS,NullS,NullS,11,11,0,0,FIELD_TYPE_LONG},
   {"NUM_RESULT_SETS",  "Catalog",NullS,NullS,NullS,11,11,0,0,FIELD_TYPE_LONG},
-  {"REMARKS",          "Catalog",NullS,NullS, NullS, NAME_LEN, 11, 0, 0, FIELD_TYPE_VAR_STRING},
+  {"REMARKS",          "Catalog",NullS,NullS,NullS,NAME_LEN,11,0,0,FIELD_TYPE_VAR_STRING},
   {"PROCEDURE_TYPE",   "Catalog",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT}
 };
+#endif
 
 const uint SQLPROCEDURES_FIELDS= array_elements(SQLPROCEDURES_values);
 uint SQLPROCEDURES_order[]= {2};
