@@ -1,4 +1,4 @@
-/* Copyright (C) 1995-2004 MySQL AB
+/* Copyright (C) 1995-2005 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -593,7 +593,7 @@ static SQLRETURN insert_fields(STMT FAR *stmt, DYNAMIC_STRING *dynQuery)
     return(SQL_ERROR);
 
   /* Get a temp result of all columns from the table .. */
-  strxmov(select, "SELECT * FROM `",stmt->table_name,"`",NullS);
+  strxmov(select, "SELECT * FROM `",stmt->table_name,"` LIMIT 0",NullS);
   MYLOG_QUERY(stmt, select);
   pthread_mutex_lock(&stmt->dbc->lock);
   if ((mysql_query(&stmt->dbc->mysql,select) ||
