@@ -1,4 +1,4 @@
-/* Copyright (C) 1995-2004 MySQL AB
+/* Copyright (C) 1995-2005 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -78,13 +78,22 @@ SQLRETURN do_query(STMT FAR *stmt,char *query)
   }
   MYLOG_QUERY(stmt, query);
   pthread_mutex_lock(&stmt->dbc->lock);
-  if (check_if_server_is_alive(stmt->dbc) ||
-      mysql_query(&stmt->dbc->mysql,query))
+  if (check_if_server_is_alive(stmt->dbc))
   {
-    set_stmt_error(stmt,"HY000",mysql_error(&stmt->dbc->mysql),
-		   mysql_errno(&stmt->dbc->mysql));
-    translate_error(stmt->error.sqlstate,MYERR_S1000,
-		    mysql_errno(&stmt->dbc->mysql));
+      set_stmt_error(stmt,"HYT00",mysql_error(&stmt->dbc->mysql),
+                     mysql_errno(&stmt->dbc->mysql));
+      translate_error(stmt->error.sqlstate,MYERR_S1000,
+              mysql_errno(&stmt->dbc->mysql));
+      goto exit;
+  }
+
+  if (mysql_query(&stmt->dbc->mysql,query))
+  {
+        set_stmt_error(stmt,"HY000",mysql_error(&stmt->dbc->mysql),
+                       mysql_errno(&stmt->dbc->mysql));
+
+        translate_error(stmt->error.sqlstate,MYERR_S1000,
+                mysql_errno(&stmt->dbc->mysql));
     goto exit;
   }
 
@@ -328,29 +337,53 @@ char *insert_param(MYSQL *mysql, char *to,PARAM_BIND *param)
   case SQL_C_BIT:
   case SQL_C_TINYINT:
   case SQL_C_STINYINT:
+#if MYSQL_VERSION_ID >= 40100
+    length= int2str((long) *((signed char*) data),buff,-10,0) -buff;
+#else
     length= int2str((long) *((signed char*) data),buff,-10) -buff;
+#endif
     data= buff;
     break;
   case SQL_C_UTINYINT:
+#if MYSQL_VERSION_ID >= 40100
+    length= int2str((long) *((unsigned char*) data),buff,-10,0) -buff;
+#else
     length= int2str((long) *((unsigned char*) data),buff,-10) -buff;
+#endif
     data= buff;
     break;
   case SQL_C_SHORT:
   case SQL_C_SSHORT:
+#if MYSQL_VERSION_ID >= 40100
+    length= int2str((long) *((short int*) data),buff,-10,0) -buff;
+#else
     length= int2str((long) *((short int*) data),buff,-10) -buff;
+#endif
     data= buff;
     break;
   case SQL_C_USHORT:
+#if MYSQL_VERSION_ID >= 40100
+    length= int2str((long) *((unsigned short int*) data),buff,-10,0) -buff;
+#else
     length= int2str((long) *((unsigned short int*) data),buff,-10) -buff;
+#endif
     data= buff;
     break;
   case SQL_C_LONG:
   case SQL_C_SLONG:
+#if MYSQL_VERSION_ID >= 40100
+    length= int2str(*((long int*) data),buff,-10,0) -buff;
+#else
     length= int2str(*((long int*) data),buff,-10) -buff;
+#endif
     data= buff;
     break;
   case SQL_C_ULONG:
+#if MYSQL_VERSION_ID >= 40100
+    length= int2str(*((long int*) data),buff,10,0) -buff;
+#else
     length= int2str(*((long int*) data),buff,10) -buff;
+#endif
     data= buff;
     break;
   case SQL_C_SBIGINT:
@@ -436,7 +469,7 @@ char *insert_param(MYSQL *mysql, char *to,PARAM_BIND *param)
     }
     else
     {
-      ulong time= str_to_time(data,length);
+      ulong time= str_to_time_as_long(data,length);
       sprintf(buff,"'%02d:%02d:%02d'",
 	      (int) time/10000,
 	      (int) time/100%100,
