@@ -1,4 +1,4 @@
-/* Copyright (C) 1995-2004 MySQL AB
+/* Copyright (C) 1995-2005 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -71,7 +71,14 @@ SQLRETURN SQL_API my_SQLAllocEnv(SQLHENV FAR *phenv)
     return (SQL_ERROR);
   }
 #endif /* _UNIX_ */
+
+/* --- if OS=WIN32, set default env option for SQL_ATTR_ODBC_VERSION */
+#ifdef WIN32
+  ((ENV FAR*) *phenv)->odbc_ver= SQL_OV_ODBC3;
+#else
   ((ENV FAR*) *phenv)->odbc_ver= SQL_OV_ODBC2;
+#endif /* WIN32 */
+
   return (SQL_SUCCESS);
 }
 
@@ -115,7 +122,7 @@ SQLRETURN SQL_API SQLFreeEnv(SQLHENV henv)
 }
 
 #ifndef _UNIX_
-SQLRETURN my_GetLastError(ENV *henv) 
+SQLRETURN my_GetLastError(ENV *henv)
 {
   SQLRETURN ret;
   LPVOID    msg;
@@ -125,13 +132,13 @@ SQLRETURN my_GetLastError(ENV *henv)
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                 (LPTSTR) &msg,
-                0, 
+                0,
                 NULL );
 
   ret = set_env_error(henv,MYERR_S1001,msg,0);
   LocalFree(msg);
 
-  return ret; 
+  return ret;
 }
 #endif
 
@@ -178,6 +185,11 @@ SQLRETURN SQL_API my_SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc)
   }
 #endif /* _UNIX_ */
 
+/* --- if OS=WIN32, set default env option for SQL_ATTR_ODBC_VERSION */
+#ifdef WIN32
+  penv->odbc_ver= SQL_OV_ODBC3;
+#endif /* WIN32 */
+
   dbc= (DBC FAR*) *phdbc;
   dbc->mysql.net.vio= 0;     /* Marker if open */
   dbc->flag= 0;
@@ -410,7 +422,7 @@ SQLRETURN SQL_API my_SQLFreeStmt(SQLHSTMT hstmt,SQLUSMALLINT fOption)
 
   if (fOption == MYSQL_RESET)
     DBUG_RETURN_STATUS(SQL_SUCCESS);
-  
+
   odbc_reset_stmt_options(&stmt->stmt_options);
 
   x_free((gptr) stmt->cursor.name);
