@@ -1,4 +1,4 @@
-/* Copyright (C) 1995-2004 MySQL AB
+/* Copyright (C) 1995-2005 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -63,14 +63,22 @@ char allowed_chars[]=
   *pcbInfoValue= sizeof(SQLUSMALLINT); \
 }
 
-#define MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue,val) \
-{ \
-  *pcbInfoValue= strlen(val); \
-  strmake((char*)rgbInfoValue,val,cbInfoValueMax); \
-  DBUG_RETURN_STATUS(*pcbInfoValue >= cbInfoValueMax ? \
-		     SQL_SUCCESS_WITH_INFO: \
-		     SQL_SUCCESS); \
-  }
+#define MYINFO_SET_STR( rgbInfoValue, cbInfoValueMax, pcbInfoValue, val )   \
+{                                                                           \
+  if ( val )                                                                \
+  {                                                                         \
+      *pcbInfoValue= strlen( val );                                         \
+      strmake( (char*)rgbInfoValue, val, cbInfoValueMax );                  \
+  }                                                                         \
+  else                                                                      \
+  {                                                                         \
+      *pcbInfoValue= 0;                                                     \
+      strmake((char*)rgbInfoValue,"",cbInfoValueMax);                       \
+  }                                                                         \
+  DBUG_RETURN_STATUS(*pcbInfoValue >= cbInfoValueMax ?                      \
+		     SQL_SUCCESS_WITH_INFO:                                         \
+		     SQL_SUCCESS);                                                  \
+}
 
 #define MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,val,len) \
 { \
@@ -358,8 +366,15 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     MYINFO_SET_STR_L(rgbInfoValue,cbInfoValueMax,pcbInfoValue,"N",1);
     break;
 
-  case SQL_DATABASE_NAME:
-    MYINFO_SET_STR(rgbInfoValue,cbInfoValueMax,pcbInfoValue,dbc->database);
+      case SQL_DATABASE_NAME:
+      {
+/*
+          char buff[80];
+          sprintf( buff,"database=%s", dbc->database );
+          DBUG_RETURN(set_conn_error(hdbc,MYERR_S1C00,buff,4000));
+*/          
+          MYINFO_SET_STR( rgbInfoValue, cbInfoValueMax, pcbInfoValue, dbc->database );
+      }
     break;
 
   case SQL_DATETIME_LITERALS:
@@ -861,46 +876,56 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   supported by mysql.
 */
 
-MYSQL_FIELD SQL_GET_TYPE_INFO_fields[]= {
-  {"TYPE_NAME","GetTypeInfo",NullS,NullS,NullS,32,32,NOT_NULL_FLAG, 0,
-   FIELD_TYPE_VAR_STRING},
-  {"DATA_TYPE","GetTypeInfo",NullS,NullS,NullS,2,2, NOT_NULL_FLAG, 0,
-   FIELD_TYPE_SHORT},
-  {"COLUMN_SIZE","GetTypeInfo",NullS,NullS,NullS,10,10,0,0,FIELD_TYPE_LONG},
-  {"LITERAL_PREFIX","GetTypeInfo",NullS,NullS,NullS,2,2,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"LITERAL_SUFFIX","GetTypeInfo",NullS,NullS,NullS,2,2,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"CREATE_PARAMS","GetTypeInfo",NullS,NullS,NullS,15,15,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"NULLABLE","GetTypeInfo",NullS,NullS,NullS,2,2,NOT_NULL_FLAG,0,
-   FIELD_TYPE_SHORT},
-  {"CASE_SENSITIVE","GetTypeInfo",NullS,NullS,NullS,2,2, NOT_NULL_FLAG, 0,
-   FIELD_TYPE_SHORT},
-  {"SEARCHABLE","GetTypeInfo",NullS,NullS,NullS,2,2, NOT_NULL_FLAG, 0,
-   FIELD_TYPE_SHORT},
-  {"UNSIGNED_ATTRIBUTE","GetTypeInfo",NullS,NullS,NullS,2,2,0,0,
-   FIELD_TYPE_SHORT},
-  {"FIXED_PREC_SCALE","GetTypeInfo",NullS,NullS,NullS,2,2, NOT_NULL_FLAG,0,
-   FIELD_TYPE_SHORT},
-  {"AUTO_UNIQUE_VALUE","GetTypeInfo",NullS,NullS,NullS,2,2,0,0,
-   FIELD_TYPE_SHORT},
-  {"LOCAL_TYPE_NAME","GetTypeInfo",NullS,NullS,NullS,60,60,0,0,
-   FIELD_TYPE_VAR_STRING},
-  {"MINIMUM_SCALE","GetTypeInfo",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
-  {"MAXIMUM_SCALE","GetTypeInfo",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
-  {"SQL_DATATYPE","GetTypeInfo",NullS,NullS,NullS,2,2,NOT_NULL_FLAG,0,
-   FIELD_TYPE_SHORT},
-  {"SQL_DATETIME_SUB","GetTypeInfo",NullS,NullS,NullS,2,2,0,0,
-   FIELD_TYPE_SHORT},
-  {"NUM_PREC_RADIX","GetTypeInfo",NullS,NullS,NullS,10,10,0,0,FIELD_TYPE_LONG},
-  {"INTERVAL_PRECISION","GetTypeInfo",NullS,NullS,NullS,2,2,0,0,
-   FIELD_TYPE_SHORT},
+#if MYSQL_VERSION_ID >= 40100
+MYSQL_FIELD SQL_GET_TYPE_INFO_fields[]= 
+{
+  {"TYPE_NAME",         NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,32,32, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_VAR_STRING},
+  {"DATA_TYPE",         NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"COLUMN_SIZE",       NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,10,10, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_LONG},
+  {"LITERAL_PREFIX",    NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"LITERAL_SUFFIX",    NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"CREATE_PARAMS",     NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,15,15, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"NULLABLE",          NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"CASE_SENSITIVE",    NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"SEARCHABLE",        NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"UNSIGNED_ATTRIBUTE",NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
+  {"FIXED_PREC_SCALE",  NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"AUTO_UNIQUE_VALUE", NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
+  {"LOCAL_TYPE_NAME",   NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,60,60, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_VAR_STRING},
+  {"MINIMUM_SCALE",     NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
+  {"MAXIMUM_SCALE",     NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
+  {"SQL_DATATYPE",      NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, NOT_NULL_FLAG,0,0,FIELD_TYPE_SHORT},
+  {"SQL_DATETIME_SUB",  NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
+  {"NUM_PREC_RADIX",    NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,10,10, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_LONG},
+  {"INTERVAL_PRECISION",NullS,"GetTypeInfo",NullS,NullS,NullS,NullS,2,2, 0,0,0,0,0,0,0, 0,0,0,FIELD_TYPE_SHORT},
 };
-
+#else
+MYSQL_FIELD SQL_GET_TYPE_INFO_fields[]= 
+{
+  {"TYPE_NAME",         "GetTypeInfo",NullS,NullS,NullS,32,32,NOT_NULL_FLAG,0,FIELD_TYPE_VAR_STRING},
+  {"DATA_TYPE",         "GetTypeInfo",NullS,NullS,NullS,2,2,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"COLUMN_SIZE",       "GetTypeInfo",NullS,NullS,NullS,10,10,0,0,FIELD_TYPE_LONG},
+  {"LITERAL_PREFIX",    "GetTypeInfo",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_VAR_STRING},
+  {"LITERAL_SUFFIX",    "GetTypeInfo",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_VAR_STRING},
+  {"CREATE_PARAMS",     "GetTypeInfo",NullS,NullS,NullS,15,15, 0,0,FIELD_TYPE_VAR_STRING},
+  {"NULLABLE",          "GetTypeInfo",NullS,NullS,NullS,2,2,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"CASE_SENSITIVE",    "GetTypeInfo",NullS,NullS,NullS,2,2,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"SEARCHABLE",        "GetTypeInfo",NullS,NullS,NullS,2,2,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"UNSIGNED_ATTRIBUTE","GetTypeInfo",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
+  {"FIXED_PREC_SCALE",  "GetTypeInfo",NullS,NullS,NullS,2,2,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"AUTO_UNIQUE_VALUE", "GetTypeInfo",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
+  {"LOCAL_TYPE_NAME",   "GetTypeInfo",NullS,NullS,NullS,60,60,0,0,FIELD_TYPE_VAR_STRING},
+  {"MINIMUM_SCALE",     "GetTypeInfo",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
+  {"MAXIMUM_SCALE",     "GetTypeInfo",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
+  {"SQL_DATATYPE",      "GetTypeInfo",NullS,NullS,NullS,2,2,NOT_NULL_FLAG,0,FIELD_TYPE_SHORT},
+  {"SQL_DATETIME_SUB",  "GetTypeInfo",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
+  {"NUM_PREC_RADIX",    "GetTypeInfo",NullS,NullS,NullS,10,10,0,0,FIELD_TYPE_LONG},
+  {"INTERVAL_PRECISION","GetTypeInfo",NullS,NullS,NullS,2,2,0,0,FIELD_TYPE_SHORT},
+};
+#endif
 
 const uint SQL_GET_TYPE_INFO_FIELDS= array_elements(SQL_GET_TYPE_INFO_fields);
-#define MYSQL_DATA_TYPES 49
+#define MYSQL_DATA_TYPES 53
 
 char sql_searchable[6],sql_unsearchable[6],sql_nullable[6],sql_no_nulls[6],
   sql_bit[6],
@@ -947,7 +972,7 @@ char *SQL_GET_TYPE_INFO_values[MYSQL_DATA_TYPES][19]=
    sql_no_nulls, "0",sql_searchable,"1","0","1",
    "bigint unsigned auto_increment",NULL,NULL,sql_bigint, NULL,"10",NULL},
 
-  /* SQL_LONGVARBINARY= -3 */
+  /* SQL_LONGVARBINARY= -4 */
   {"long varbinary",sql_longvarbinary,"16777215","0x",NULL,NULL,sql_nullable,
    "0",sql_searchable,"0","0","0","mediumblob",NULL,NULL,sql_longvarbinary,
    NULL,NULL,NULL},
@@ -958,13 +983,24 @@ char *SQL_GET_TYPE_INFO_values[MYSQL_DATA_TYPES][19]=
    "0",sql_searchable,"0","0","0",
    "binary large object, use mediumblob instead",NULL,NULL,
    sql_longvarbinary,NULL,NULL,NULL},
-  {"tinyblob",sql_varbinary,"255","'","'",NULL,sql_nullable,
+  {"tinyblob",sql_longvarbinary,"255","'","'",NULL,sql_nullable,
    "0",sql_searchable,"0","0","0","binary large object (0-255) ",NULL,NULL,
-   sql_varbinary,NULL,NULL,NULL},
+   sql_longvarbinary,NULL,NULL,NULL},
   {"mediumblob",sql_longvarbinary,"16777215","'","'",NULL,sql_nullable,
    "0",sql_searchable,"0","0","0","binary large object",NULL,NULL,
    sql_longvarbinary,NULL,NULL,NULL},
 
+  /* SQL_VARBINARY= -3 */
+  {"varbinary",sql_varbinary,"255","'","'","length",sql_nullable,
+   "0",sql_searchable,"0","0","0","varbinary",NULL,NULL,sql_varbinary,
+   NULL,NULL,NULL},
+
+  /* SQL_BINARY= -2 */
+  {"binary",sql_binary,"255","'","'","length",sql_nullable,
+   "0",sql_searchable,"0","0","0","binary",NULL,NULL,sql_binary,
+   NULL,NULL,NULL},
+
+
   /* SQL_LONGVARCHAR= -1 */
   {"long varchar",sql_longvarchar,"16777215","'","'",NULL,sql_nullable,
    "0",sql_searchable,"0","0","0","mediumtext",NULL,NULL,
@@ -975,18 +1011,26 @@ char *SQL_GET_TYPE_INFO_values[MYSQL_DATA_TYPES][19]=
   {"mediumtext",sql_longvarchar,"16777215","'","'",NULL,sql_nullable,
    "0",sql_searchable,"0","0","0","mediumtext",NULL,NULL,
    sql_longvarchar,NULL,NULL,NULL},
+  {"longtext",sql_longvarchar,"2147483647","'","'",NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","longtext",NULL,NULL,
+   sql_longvarchar,NULL,NULL,NULL},
+  {"tinytext",sql_longvarchar,"255","'","'",NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","tinytext",NULL,NULL,
+   sql_longvarchar,NULL,NULL,NULL},
 
   /* SQL_CHAR= 1 */
   {"char",sql_char,"255","'","'","length",sql_nullable,
    "0",sql_searchable,"0","0","0","char",NULL,NULL,sql_char,
    NULL,NULL,NULL},
 
-  /* SQL_DECIMAL= 3 */
+  /* SQL_NUMERIC= 2 */
   {"numeric",sql_numeric,"19",NULL,NULL,"precision,scale",sql_nullable,
    "0",sql_searchable,"0","0","0","numeric","0","19",sql_numeric,
    NULL,"10",NULL},
+
+  /* SQL_DECIMAL= 3 */
   {"decimal",sql_decimal,"19",NULL,NULL,"precision,scale",sql_nullable,
-   "0",sql_searchable,"0","0","0","decimal","0","19",sql_numeric,
+   "0",sql_searchable,"0","0","0","decimal","0","19",sql_decimal,
    NULL,"10",NULL},
 
   /* SQL_INTEGER= 4 */
@@ -1160,6 +1204,39 @@ SQLRETURN SQL_API SQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT fSqlType)
 void init_getfunctions(void)
 {
   /* Make some integers to strings for easy init of string arrays */
+#if MYSQL_VERSION_ID >= 40100
+  int2str(SQL_SEARCHABLE,sql_searchable,-10,0);
+  int2str(SQL_UNSEARCHABLE,sql_unsearchable,-10,0);
+  int2str(SQL_NULLABLE,sql_nullable,-10,0);
+  int2str(SQL_NO_NULLS,sql_no_nulls,-10,0);
+  int2str(SQL_BIT,sql_bit,-10,0);
+  int2str(SQL_TINYINT,sql_tinyint,-10,0);
+  int2str(SQL_SMALLINT,sql_smallint,-10,0);
+  int2str(SQL_INTEGER,sql_integer,-10,0);
+  int2str(SQL_BIGINT,sql_bigint,-10,0);
+  int2str(SQL_DECIMAL,sql_decimal,-10,0);
+  int2str(SQL_NUMERIC,sql_numeric,-10,0);
+  int2str(SQL_REAL,sql_real,-10,0);
+  int2str(SQL_FLOAT,sql_float,-10,0);
+  int2str(SQL_DOUBLE,sql_double,-10,0);
+  int2str(SQL_CHAR,sql_char,-10,0);
+  int2str(SQL_VARCHAR,sql_varchar,-10,0);
+  int2str(SQL_LONGVARCHAR,sql_longvarchar,-10,0);
+  int2str(SQL_LONGVARBINARY,sql_longvarbinary,-10,0);
+  int2str(SQL_VARBINARY,sql_varbinary,-10,0);
+  int2str(SQL_BINARY,sql_binary,-10,0);
+#if (ODBCVER < 0x0300)
+  int2str(SQL_TIMESTAMP,sql_timestamp,-10,0);
+  int2str(SQL_DATE,sql_date,-10,0);
+  int2str(SQL_TIME,sql_time,-10,0);
+  myodbc_sqlstate2_init();
+  myodbc_ov2_inited= 1;
+#else
+  int2str(SQL_TYPE_TIMESTAMP,sql_timestamp,-10,0);
+  int2str(SQL_TYPE_DATE,sql_date,-10,0);
+  int2str(SQL_TYPE_TIME,sql_time,-10,0);
+#endif
+#else
   int2str(SQL_SEARCHABLE,sql_searchable,-10);
   int2str(SQL_UNSEARCHABLE,sql_unsearchable,-10);
   int2str(SQL_NULLABLE,sql_nullable,-10);
@@ -1180,7 +1257,6 @@ void init_getfunctions(void)
   int2str(SQL_LONGVARBINARY,sql_longvarbinary,-10);
   int2str(SQL_VARBINARY,sql_varbinary,-10);
   int2str(SQL_BINARY,sql_binary,-10);
-
 #if (ODBCVER < 0x0300)
   int2str(SQL_TIMESTAMP,sql_timestamp,-10);
   int2str(SQL_DATE,sql_date,-10);
@@ -1192,6 +1268,8 @@ void init_getfunctions(void)
   int2str(SQL_TYPE_DATE,sql_date,-10);
   int2str(SQL_TYPE_TIME,sql_time,-10);
 #endif
+#endif
+
 }
 
 /*
@@ -1203,9 +1281,15 @@ void myodbc_ov_init(SQLINTEGER odbc_version)
 {
   if (odbc_version == SQL_OV_ODBC2)
   {
+#if MYSQL_VERSION_ID >= 40100
+    int2str(SQL_TIMESTAMP,sql_timestamp,-10,0);
+    int2str(SQL_DATE,sql_date,-10,0);
+    int2str(SQL_TIME,sql_time,-10,0);
+#else
     int2str(SQL_TIMESTAMP,sql_timestamp,-10);
     int2str(SQL_DATE,sql_date,-10);
     int2str(SQL_TIME,sql_time,-10);
+#endif
     myodbc_sqlstate2_init();
     myodbc_ov2_inited= 1;
   }
@@ -1215,9 +1299,15 @@ void myodbc_ov_init(SQLINTEGER odbc_version)
       return;
     myodbc_ov2_inited= 0;
 
+#if MYSQL_VERSION_ID >= 40100
+    int2str(SQL_TYPE_TIMESTAMP,sql_timestamp,-10,0);
+    int2str(SQL_TYPE_DATE,sql_date,-10,0);
+    int2str(SQL_TYPE_TIME,sql_time,-10,0);
+#else
     int2str(SQL_TYPE_TIMESTAMP,sql_timestamp,-10);
     int2str(SQL_TYPE_DATE,sql_date,-10);
     int2str(SQL_TYPE_TIME,sql_time,-10);
+#endif
     myodbc_sqlstate3_init();
   }
 }
