@@ -1,4 +1,4 @@
-/* Copyright (C) 1995-2004 MySQL AB
+/* Copyright (C) 1995-2005 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -126,7 +126,7 @@ int default_c_type(int sql_data_type);
 ulong bind_length(int sql_data_type,ulong length);
 my_bool str_to_date(SQL_DATE_STRUCT *rgbValue, const char *str,uint length);
 my_bool str_to_ts(SQL_TIMESTAMP_STRUCT *ts, const char *str);
-ulong str_to_time(const char *str,uint length);
+ulong str_to_time_as_long(const char *str,uint length);
 void init_getfunctions(void);
 void myodbc_init(void);
 void myodbc_ov_init(SQLINTEGER odbc_version);
