@@ -1,4 +1,4 @@
-/* Copyright (C) 1995-2004 MySQL AB
+/* Copyright (C) 1995-2005 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -274,140 +274,146 @@ get_constmt_attr(SQLSMALLINT  HandleType,
 */
 
 static SQLRETURN
-set_con_attr(SQLHDBC	hdbc,
-	     SQLINTEGER Attribute,
-	     SQLPOINTER ValuePtr,
-	     SQLINTEGER StringLengthPtr)
+set_con_attr(SQLHDBC    hdbc,
+             SQLINTEGER Attribute,
+             SQLPOINTER ValuePtr,
+             SQLINTEGER StringLengthPtr)
 {
-  DBC FAR *dbc= (DBC FAR*) hdbc;
-  DBUG_ENTER("SQLSetConnectAttr");
-  DBUG_PRINT("enter",("Atrr: %d, ValuePtr: 0x%lx, StrLenPtr: %d",Attribute,ValuePtr,StringLengthPtr));
+    DBC FAR *dbc= (DBC FAR*) hdbc;
+    DBUG_ENTER("SQLSetConnectAttr");
+    DBUG_PRINT("enter",("Atrr: %d, ValuePtr: 0x%lx, StrLenPtr: %d",Attribute,ValuePtr,StringLengthPtr));
 
-  switch(Attribute) {
-
-  case SQL_ATTR_ACCESS_MODE:
-    DBUG_PRINT("info",("SQL_ATTR_ACCESS_MODE %ld ignored",
-		       (SQLUINTEGER)ValuePtr));
-    break;
-
-  case SQL_ATTR_AUTOCOMMIT:
-    if ((SQLUINTEGER)ValuePtr != SQL_AUTOCOMMIT_ON)
+    switch (Attribute)
     {
-      if (!dbc->server) /* no connection yet */
-      {
-	dbc->commit_flag= CHECK_AUTOCOMMIT_OFF;
-	DBUG_RETURN_STATUS(SQL_SUCCESS);
-      }
-      if (!(trans_supported(dbc)) || (dbc->flag & FLAG_NO_TRANSACTIONS))
-	DBUG_RETURN(set_conn_error(dbc,MYERR_S1C00,
-				   "Transactions are not enabled", 4000));
+        
+        case SQL_ATTR_ACCESS_MODE:
+            DBUG_PRINT("info",("SQL_ATTR_ACCESS_MODE %ld ignored",
+                               (SQLUINTEGER)ValuePtr));
+            break;
 
-      if (autocommit_on(dbc))
-	DBUG_RETURN(odbc_stmt(dbc,"SET AUTOCOMMIT=0"));
+        case SQL_ATTR_AUTOCOMMIT:
+            if ((SQLUINTEGER)ValuePtr != SQL_AUTOCOMMIT_ON)
+            {
+                if (!dbc->server) /* no connection yet */
+                {
+                    dbc->commit_flag= CHECK_AUTOCOMMIT_OFF;
+                    DBUG_RETURN_STATUS(SQL_SUCCESS);
+                }
+                if (!(trans_supported(dbc)) || (dbc->flag & FLAG_NO_TRANSACTIONS))
+                    DBUG_RETURN(set_conn_error(dbc,MYERR_S1C00,
+                                               "Transactions are not enabled", 4000));
+
+                if (autocommit_on(dbc))
+                    DBUG_RETURN(odbc_stmt(dbc,"SET AUTOCOMMIT=0"));
+            }
+            else if (!dbc->server) /* no connection yet */
+            {
+                dbc->commit_flag= CHECK_AUTOCOMMIT_ON;
+                DBUG_RETURN_STATUS(SQL_SUCCESS);
+            }
+            else if (trans_supported(dbc) && !(autocommit_on(dbc)))
+                DBUG_RETURN(odbc_stmt(dbc,"SET AUTOCOMMIT=1"));
+            break;
+
+        case SQL_ATTR_CONNECTION_TIMEOUT:
+            {
+                uint timeout= (SQLUINTEGER)ValuePtr;
+                DBUG_RETURN(mysql_options(&dbc->mysql, MYSQL_OPT_CONNECT_TIMEOUT,
+                                          (const char *)&timeout));
+            }
+            break;
+
+            /*
+                  If this is done before connect (I would say a function 
+                  sequence but .NET IDE does this) then we store the value but
+                  it is quite likely that it will get replaced by DATABASE in
+                  a DSN or connect string.
+            */
+        case SQL_ATTR_CURRENT_CATALOG:
+            {
+                char ldb[NAME_LEN+1], *db;
+
+                if (!(db= fix_str((char *)ldb, (char *)ValuePtr, StringLengthPtr)))
+                    DBUG_RETURN(set_conn_error(hdbc,MYERR_S1009,NULL, 0));
+
+                pthread_mutex_lock(&dbc->lock);
+                if ( dbc->mysql.net.vio )
+                {
+                    if (mysql_select_db(&dbc->mysql,(char*) db))
+                    {
+                        set_conn_error(dbc,MYERR_S1000,mysql_error(&dbc->mysql),mysql_errno(&dbc->mysql));
+                        pthread_mutex_unlock(&dbc->lock);
+                        DBUG_RETURN_STATUS(SQL_ERROR);
+                    }
+                }
+                my_free((gptr) dbc->database,MYF(0));
+                dbc->database= my_strdup(db,MYF(MY_WME));
+                pthread_mutex_unlock(&dbc->lock);
+            }
+            break;
+
+        case SQL_ATTR_LOGIN_TIMEOUT:
+            dbc->login_timeout= (SQLUINTEGER)ValuePtr;
+            break;
+
+        case SQL_ATTR_ODBC_CURSORS:
+            if ((dbc->flag & FLAG_FORWARD_CURSOR) &&
+                (SQLUINTEGER)ValuePtr != SQL_CUR_USE_ODBC)
+                DBUG_RETURN(set_conn_error(hdbc,MYERR_01S02,
+                                           "Forcing the Driver Manager to use ODBC cursor library",0));
+            break;
+
+        case SQL_OPT_TRACE:
+        case SQL_OPT_TRACEFILE:
+        case SQL_QUIET_MODE:
+        case SQL_TRANSLATE_DLL:
+        case SQL_TRANSLATE_OPTION:
+            {
+                char buff[100];
+                sprintf(buff,"Suppose to set this attribute '%d' through driver manager, not by the driver",(int) Attribute);
+                DBUG_RETURN(set_conn_error(hdbc,MYERR_01S02,buff,0));
+            }
+
+        case SQL_ATTR_PACKET_SIZE:
+            DBUG_PRINT("info",("SQL_ATTR_PACKET_SIZE %ld ignored",(SQLUINTEGER)ValuePtr));
+            break;
+
+        case SQL_ATTR_TXN_ISOLATION:
+            if (!dbc->server)  /* no connection yet */
+            {
+                dbc->txn_isolation= (SQLINTEGER)ValuePtr;
+                DBUG_RETURN_STATUS(SQL_SUCCESS);
+            }
+            if (trans_supported(dbc))
+            {
+                char buff[80];
+                const char *level;
+
+                if ((SQLINTEGER)ValuePtr & SQL_TXN_SERIALIZABLE)
+                    level="SERIALIZABLE";
+                else if ((SQLINTEGER)ValuePtr & SQL_TXN_REPEATABLE_READ)
+                    level="REPEATABLE READ";
+                else if ((SQLINTEGER)ValuePtr & SQL_TXN_REPEATABLE_READ)
+                    level="READ COMMITTED";
+                else
+                    level="READ UNCOMMITTED";
+                sprintf(buff,"SET SESSION TRANSACTION ISOLATION LEVEL %s",level);
+                if (odbc_stmt(dbc,buff) == SQL_SUCCESS)
+                    dbc->txn_isolation= (SQLINTEGER)ValuePtr;
+            }
+            else
+                DBUG_PRINT("info",("SQL_ATTR_TXN_ISOLATION %ld ignored",(SQLINTEGER)ValuePtr));
+            break;
+
+            /*
+              3.x driver doesn't support any statement attributes
+              at connection level, but to make sure all 2.x apps
+              works fine...lets support it..nothing to loose..
+            */
+        default:
+            DBUG_RETURN(set_constmt_attr(2,dbc,&dbc->stmt_options,Attribute,ValuePtr));
     }
-    else if (!dbc->server) /* no connection yet */
-    {
-      dbc->commit_flag= CHECK_AUTOCOMMIT_ON;
-      DBUG_RETURN_STATUS(SQL_SUCCESS);
-    }
-    else if (trans_supported(dbc) && !(autocommit_on(dbc)))
-      DBUG_RETURN(odbc_stmt(dbc,"SET AUTOCOMMIT=1"));
-    break;
-
-  case SQL_ATTR_CONNECTION_TIMEOUT:
-  {
-    uint timeout= (SQLUINTEGER)ValuePtr;
-    DBUG_RETURN(mysql_options(&dbc->mysql, MYSQL_OPT_CONNECT_TIMEOUT,
-			      (const char *)&timeout));
-  }
-  break;
-
-  case SQL_ATTR_CURRENT_CATALOG:
-  {
-    char ldb[NAME_LEN+1], *db;
-    
-    if (!(db= fix_str((char *)ldb, (char *)ValuePtr, StringLengthPtr)))
-      DBUG_RETURN(set_conn_error(hdbc,MYERR_S1009,NULL, 0));
-
-    pthread_mutex_lock(&dbc->lock);
-    if (mysql_select_db(&dbc->mysql,(char*) db))
-    {
-      set_conn_error(dbc,MYERR_S1000,mysql_error(&dbc->mysql),
-		     mysql_errno(&dbc->mysql));
-      pthread_mutex_unlock(&dbc->lock);
-      DBUG_RETURN_STATUS(SQL_ERROR);
-    }
-    my_free((gptr) dbc->database,MYF(0));
-    dbc->database= my_strdup(db,MYF(MY_WME));
-    pthread_mutex_unlock(&dbc->lock);
-  }
-  break;
-
-  case SQL_ATTR_LOGIN_TIMEOUT:
-    dbc->login_timeout= (SQLUINTEGER)ValuePtr;
-    break;
-
-  case SQL_ATTR_ODBC_CURSORS:
-    if ((dbc->flag & FLAG_FORWARD_CURSOR) &&
-	(SQLUINTEGER)ValuePtr != SQL_CUR_USE_ODBC)
-      DBUG_RETURN(set_conn_error(hdbc,MYERR_01S02,
-				 "Forcing the Driver Manager to use ODBC cursor library",0));
-    break;
-
-  case SQL_OPT_TRACE:
-  case SQL_OPT_TRACEFILE:
-  case SQL_QUIET_MODE:
-  case SQL_TRANSLATE_DLL:
-  case SQL_TRANSLATE_OPTION:
-  {
-    char buff[100];
-    sprintf(buff,"Suppose to set this attribute '%d' through driver manager, not by the driver",(int) Attribute);
-    DBUG_RETURN(set_conn_error(hdbc,MYERR_01S02,buff,0));
-  }
-
-  case SQL_ATTR_PACKET_SIZE:
-    DBUG_PRINT("info",("SQL_ATTR_PACKET_SIZE %ld ignored",
-		       (SQLUINTEGER)ValuePtr));
-    break;
-
-  case SQL_ATTR_TXN_ISOLATION:
-    if (!dbc->server)  /* no connection yet */
-    {
-      dbc->txn_isolation= (SQLINTEGER)ValuePtr;
-      DBUG_RETURN_STATUS(SQL_SUCCESS);
-    }
-    if (trans_supported(dbc))
-    {
-      char buff[80];
-      const char *level;
-
-      if ((SQLINTEGER)ValuePtr & SQL_TXN_SERIALIZABLE)
-	level="SERIALIZABLE";
-      else if ((SQLINTEGER)ValuePtr & SQL_TXN_REPEATABLE_READ)
-	level="REPEATABLE READ";
-      else if ((SQLINTEGER)ValuePtr & SQL_TXN_REPEATABLE_READ)
-	level="READ COMMITTED";
-      else
-	level="READ UNCOMMITTED";
-      sprintf(buff,"SET SESSION TRANSACTION ISOLATION LEVEL %s",level);
-      if (odbc_stmt(dbc,buff) == SQL_SUCCESS)
-	dbc->txn_isolation= (SQLINTEGER)ValuePtr;
-    }
-    else
-      DBUG_PRINT("info",("SQL_ATTR_TXN_ISOLATION %ld ignored",
-			 (SQLINTEGER)ValuePtr));
-    break;
-
-    /*
-      3.x driver doesn't support any statement attributes
-      at connection level, but to make sure all 2.x apps
-      works fine...lets support it..nothing to loose..
-    */
-  default:
-    DBUG_RETURN(set_constmt_attr(2,dbc,&dbc->stmt_options,
-				 Attribute,ValuePtr));
-  }
-  DBUG_RETURN_STATUS(SQL_SUCCESS);
+    DBUG_RETURN_STATUS(SQL_SUCCESS);
 }
 
 
@@ -469,13 +475,18 @@ static SQLRETURN get_con_attr(SQLHDBC	 hdbc,
 #endif
     break;
 
-  case SQL_ATTR_CURRENT_CATALOG:
+    case SQL_ATTR_CURRENT_CATALOG:
+
     if (reget_current_catalog(dbc))
-      result= SQL_ERROR;
+    {
+        result= SQL_ERROR;
+    }
     else
-      *StringLengthPtr= (SQLSMALLINT) (strmake((char*)ValuePtr,dbc->database,
-					       BufferLength) -
-				       (char*) ValuePtr);
+    {
+        *StringLengthPtr= (SQLSMALLINT) (strmake((char*)ValuePtr,dbc->database,
+                             BufferLength) -
+                         (char*) ValuePtr);
+    }
     break;
 
   case SQL_ATTR_LOGIN_TIMEOUT:
@@ -837,29 +848,33 @@ SQLSetEnvAttr(SQLHENV	 henv,
 */
 
 SQLRETURN SQL_API
-SQLGetEnvAttr(SQLHENV	 henv,
-	      SQLINTEGER Attribute,
-	      SQLPOINTER ValuePtr,
-	      SQLINTEGER BufferLength __attribute__((unused)),
-	      SQLINTEGER *StringLengthPtr __attribute__((unused)))
+SQLGetEnvAttr(SQLHENV    henv,
+              SQLINTEGER Attribute,
+              SQLPOINTER ValuePtr,
+              SQLINTEGER BufferLength __attribute__((unused)),
+              SQLINTEGER *StringLengthPtr __attribute__((unused)))
 {
-  DBUG_ENTER("SQLGetEnvAttr");
-  DBUG_PRINT("enter",("Atrr: %d",Attribute));
+    DBUG_ENTER("SQLGetEnvAttr");
+    DBUG_PRINT("enter",("Atrr: %d",Attribute));
 
-  switch(Attribute) {
+    switch ( Attribute )
+    {
+        case SQL_ATTR_CONNECTION_POOLING:
+            *(SQLINTEGER*)ValuePtr = SQL_CP_OFF;
+            break;
 
-  case SQL_ATTR_ODBC_VERSION:
-    *(SQLINTEGER*)ValuePtr= ((ENV FAR *)henv)->odbc_ver;
-    break;
+        case SQL_ATTR_ODBC_VERSION:
+            *(SQLINTEGER*)ValuePtr= ((ENV FAR *)henv)->odbc_ver;
+            break;
 
-  case SQL_ATTR_OUTPUT_NTS:
-    *((SQLINTEGER*)ValuePtr)= SQL_TRUE;
-    break;
+        case SQL_ATTR_OUTPUT_NTS:
+            *((SQLINTEGER*)ValuePtr)= SQL_TRUE;
+            break;
 
-  default:
-    DBUG_RETURN(set_env_error(henv,MYERR_S1C00,NULL,0));
-  }
-  DBUG_RETURN_STATUS(SQL_SUCCESS);
+        default:
+            DBUG_RETURN(set_env_error(henv,MYERR_S1C00,NULL,0));
+    }
+    DBUG_RETURN_STATUS(SQL_SUCCESS);
 }
 
 
