@@ -1,4 +1,4 @@
-/* Copyright (C) 1995-2004 MySQL AB
+/* Copyright (C) 1995-2005 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,6 +33,11 @@
 #include "errmsg.h"
 #include "m_ctype.h"
 
+#if MYSQL_VERSION_ID >= 40100
+#undef USE_MB
+#endif
+
+
 /*
   @type    : myodbc internal
   @purpose : executes the specified sql statement
@@ -55,7 +60,6 @@ SQLRETURN odbc_stmt(DBC FAR *dbc, const char *query)
   DBUG_RETURN(result);
 }
 
-
 /*
   @type    : myodbc internal
   @purpose : use own fields instead of sql fields
@@ -306,6 +310,10 @@ copy_binary_result(SQLSMALLINT HandleType, SQLHANDLE Handle,
 {
   char *dst= (char*) rgbValue;
   ulong length;
+#if MYSQL_VERSION_ID >= 40100
+  char NEAR _dig_vec[] =
+  "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+#endif
 
   if (!cbValueMax)
     dst= 0;	 /* Don't copy anything! */
@@ -815,7 +823,7 @@ my_bool str_to_date(SQL_DATE_STRUCT *rgbValue, const char *str,uint length)
   @return  : HHMMSS
 */
 
-ulong str_to_time(const char *str,uint length)
+ulong str_to_time_as_long(const char *str,uint length)
 {
   uint i,date[3];
   const char *end= str+length;
@@ -844,7 +852,7 @@ ulong str_to_time(const char *str,uint length)
     }
   }
   if (length && str != end)    
-    return str_to_time(str, length);/* timestamp format */
+    return str_to_time_as_long(str, length);/* timestamp format */
 
   if (date[0] > 10000L || i < 3)	/* Properly handle HHMMSS format */
     return (ulong) date[0];
@@ -902,22 +910,31 @@ my_bool dynstr_append_quoted_name(DYNAMIC_STRING *str, const char *name)
 
 my_bool reget_current_catalog(DBC FAR *dbc)
 {
-  if (odbc_stmt(dbc, "select database()"))
-    return 1;
-  else
-  {
-    MYSQL_RES *res;
-    MYSQL_ROW row;
-
-    if ((res= mysql_store_result(&dbc->mysql)) &&
-	(row= mysql_fetch_row(res)))
+    my_free((gptr) dbc->database,MYF(0));
+    if (odbc_stmt(dbc, "select database()"))
     {
-      if (cmp_database(row[0], dbc->database))
-	dbc->database= my_strdup(row[0], MYF(MY_WME));
+        return 1;
     }
-    mysql_free_result(res);
-  }
-  return 0;
+    else
+    {
+        MYSQL_RES *res;
+        MYSQL_ROW row;
+
+        if ((res= mysql_store_result(&dbc->mysql)) &&
+            (row= mysql_fetch_row(res)))
+        {
+/*            if (cmp_database(row[0], dbc->database)) */
+            {
+                if ( row[0] )
+                    dbc->database = my_strdup(row[0], MYF(MY_WME));
+                else
+                    dbc->database = strdup( "null" );
+            }
+        }
+        mysql_free_result(res);
+    }
+
+    return 0;
 }
 
 
