@@ -90,7 +90,7 @@ cd ${RPM_BUILD_AREA}/SPECS
 
 echo "Running the spec file from '${RPM_BUILD_AREA}/SPECS'"
 
-rpm -ba ${SPEC_FILE}
+rpmbuild -ba ${SPEC_FILE}
 if test $? -eq 0 
 then
   ln -s ${RPM_BUILD_AREA}/RPMS/${ARCH}/$SRC_FILE-1.${ARCH}.rpm $SRC_FILE-1.${ARCH}.rpm
